package cc.winboll.studio.webpagesources.common;

/**
 * @Author ZhanGSKen@QQ.COM
 * @Date 2023/06/22 12:57:08
 * @Describe 网页客户端视图基础类
 * 参考：https://blog.csdn.net/Azhuoyanyan/article/details/17531887?app_version=5.15.2&code=app_1562916241&csdn_share_tail=%7B%22type%22%3A%22blog%22%2C%22rType%22%3A%22article%22%2C%22rId%22%3A%2217531887%22%2C%22source%22%3A%22weixin_38986226%22%7D&uLinkId=usr1mkqgl919blen&utm_source=app
 *
 */
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.net.http.SslError;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.DownloadListener;
import android.webkit.HttpAuthHandler;
import android.webkit.JavascriptInterface;
import android.webkit.SslErrorHandler;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebResourceError;
import android.webkit.WebResourceRequest;
import android.webkit.WebResourceResponse;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.view.GestureDetectorCompat;
import cc.winboll.studio.libappbase.LogUtils;
import cc.winboll.studio.webpagesources.MainActivity;
import cc.winboll.studio.webpagesources.R;
import cc.winboll.studio.webpagesources.thread.LinkDownLoadThread;
import cc.winboll.studio.webpagesources.util.UIUtil;
import cc.winboll.studio.webpagesources.view.ItemLongClickedPopWindow;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class BaseWebView extends WebView {

    public static final String TAG = "BaseWebView";

    public static final int WEB_CopyLink = 0;
    public static final int WEB_DownLoadLink = 1;

    protected Context mContext;
    static String _mszLastUrl = "";
    IOnPageFinished mIOnPageFinished;
    protected MyWebChromeClient mMyWebChromeClient;
    protected BaseWebViewClient mBaseWebViewClient;
    protected BaseWebViewHandler mBaseWebViewHandler;
    private int downX, downY;
    private GestureDetectorCompat mGestureDetector;
    ValueCallback<Uri[]> mFilePathCallback;
    WebChromeClient.FileChooserParams mFileChooserParams;

    AuthLoginDialog mAuthLoginDialog;
    // AuthenticationBean mLastAuthenticationBean;

    /**
     * 在java代码里new的时候会用到
     * @param context
     */
    public BaseWebView(Context context) {
        super(context);
    }

    /**
     * 在xml布局文件中使用时自动调用
     * @param context
     */
    public BaseWebView(Context context, AttributeSet attrs) {
        super(context, attrs);
        mContext = context;

        mBaseWebViewHandler = new BaseWebViewHandler();
        /*TypedArray ta = context.obtainStyledAttributes(attrs, R.styleable.test);
         String text = ta.getString(R.styleable.test_text);
         setText(text + " ZhanGSKen.CN");
         ta.recycle();*/
        //setBackgroundResource(R.drawable.custom_edittext_background);
        //setMinWidth(100);
        //setPadding(10,0,10,0);
        //setPaddingRelative(0,0,0,0);
        MyWebChromeClient mMyWebChromeClient = new MyWebChromeClient();
        setWebChromeClient(mMyWebChromeClient);
        mBaseWebViewClient = new BaseWebViewClient();
        setWebViewClient(mBaseWebViewClient);

        WebSettings mWebSettings = getSettings();
        // 设置是否启用安全浏览。
        mWebSettings.setSafeBrowsingEnabled(true);
        // 启用JavaScript
        mWebSettings.setJavaScriptEnabled(true);
        // 设置可以支持缩放        
        mWebSettings.setSupportZoom(true);        
        // 设置true,才能让Webivew支持<meta>标签的viewport属性
        mWebSettings.setUseWideViewPort(true);        
        // 设置出现缩放工具        
        mWebSettings.setBuiltInZoomControls(true);        
        // 设置隐藏缩放控件        
        mWebSettings.setDisplayZoomControls(false);
        mWebSettings.setDefaultTextEncodingName("utf-8");
        // 允许WebView运行JavaScript，并开启JavaScript Console以接收日志
        mWebSettings.setDomStorageEnabled(true); // 如果有localStorage等需要
        mWebSettings.setJavaScriptCanOpenWindowsAutomatically(true);
        addJavascriptInterface(new JSConsole(mContext), "console");
        addJavascriptInterface(new JS(mContext), "local_obj");

        // 最小缩放等级        
        setInitialScale(60);

        mWebSettings.setJavaScriptCanOpenWindowsAutomatically(true);
        //winbollView.loadUrl("https://www.so.com");

        mGestureDetector = new GestureDetectorCompat(mContext, new GestureDetector.SimpleOnGestureListener() {
                @Override
                public void onLongPress(MotionEvent e) {
                    downX = (int) e.getX();
                    downY = (int) e.getY();
                    //LogUtils.d(TAG, "downX is " + Integer.toString(downX));
                    //LogUtils.d(TAG, "downY is " + Integer.toString(downY));
                }
            });

        setOnLongClickListener(new View.OnLongClickListener() {
                String szUrl = "";
                @Override
                public boolean onLongClick(View v) {

                    WebView.HitTestResult result = ((WebView)v).getHitTestResult();
                    if (null == result) 
                        return false;
                    int type = result.getType();
                    if (type == WebView.HitTestResult.UNKNOWN_TYPE) 
                        return false;
                    if (type == WebView.HitTestResult.EDIT_TEXT_TYPE) {
                        //let TextViewhandles context menu return true;
                    }
                    final ItemLongClickedPopWindow itemLongClickedPopWindow = new ItemLongClickedPopWindow(mContext, ItemLongClickedPopWindow.IMAGE_VIEW_POPUPWINDOW, UIUtil.dip2px(mContext, 180), ViewGroup.LayoutParams.WRAP_CONTENT);
                    // Setup custom handlingdepending on the type
                    switch (type) {
                        case WebView.HitTestResult.PHONE_TYPE: // 处理拨号
                            break;
                        case WebView.HitTestResult.EMAIL_TYPE: // 处理Email
                            break;
                        case WebView.HitTestResult.GEO_TYPE: // TODO
                            break;

                        case WebView.HitTestResult.SRC_IMAGE_ANCHOR_TYPE:
                            break;
                        case WebView.HitTestResult.SRC_ANCHOR_TYPE: // 超链接
                            // Log.d(DEG_TAG, "超链接");
                            //szUrl = result.getExtra();
                            //break;
                        case WebView.HitTestResult.IMAGE_TYPE: // 处理长按图片的菜单项
                            szUrl = result.getExtra();
                            //通过GestureDetector获取按下的位置，来定位PopWindow显示的位置
                            itemLongClickedPopWindow.showAtLocation(v, Gravity.TOP | Gravity.LEFT, downX, downY + 10);
                            break;
                        default:
                            break;
                    }

                    itemLongClickedPopWindow.getView(R.id.item_longclicked_copylink)
                        .setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                // 复制链接到剪贴板
                                // Gets a handle to the clipboard service.
                                ClipboardManager clipboard = (ClipboardManager) mContext.getSystemService(Context.CLIPBOARD_SERVICE);
                                // Creates a new text clip to put on the clipboard
                                ClipData clip = ClipData.newPlainText("simple text", szUrl);
                                // Set the clipboard's primary clip.
                                clipboard.setPrimaryClip(clip);
                                Toast.makeText(mContext, "Copy to clipboard.", Toast.LENGTH_SHORT).show();
                                itemLongClickedPopWindow.dismiss();
                            }
                        });
                    itemLongClickedPopWindow.getView(R.id.item_longclicked_downloadlink)
                        .setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                itemLongClickedPopWindow.dismiss();
                                new Thread(new LinkDownLoadThread(szUrl)).start();
                            }
                        });
                    return true;
                }
            });
        /*setOnCreateContextMenuListener(new View.OnCreateContextMenuListener(){
         public void onCreateContextMenu(ContextMenu menu, View arg1,
         ContextMenuInfo arg2) {
         final HitTestResult hitTestResult = ((WebView) arg1).getHitTestResult();

         MenuItem.OnMenuItemClickListener handler = new MenuItem.OnMenuItemClickListener() {

         public boolean onMenuItemClick(MenuItem item) {
         switch (item.getItemId()) {
         case WEB_CopyLink : {

         break;
         }
         case WEB_DownLoadLink : {
         // 下载链接地址
         new Thread(new LinkDownLoadThread(hitTestResult.getExtra())).start();
         break;
         }
         }
         return true;
         }
         };

         // 添加上下文菜单
         int resultType = hitTestResult.getType();
         if ((resultType == HitTestResult.ANCHOR_TYPE) ||
         (resultType == HitTestResult.IMAGE_ANCHOR_TYPE) ||
         (resultType == HitTestResult.SRC_ANCHOR_TYPE) ||
         (resultType == HitTestResult.SRC_IMAGE_ANCHOR_TYPE)) {
         Intent i = new Intent();
         //AppLog.d("extrea", result.getExtra());
         MenuItem item = menu.add(0, WEB_CopyLink, 0, "Copy Link").setOnMenuItemClickListener(handler);
         MenuItem item2 = menu.add(0, WEB_DownLoadLink, 0, "DownLoad Link").setOnMenuItemClickListener(handler);
         item.setIntent(i);
         //menu.setHeaderTitle(hitTestResult.getExtra());
         TextView tv = new TextView(mContext);
         tv.setText(hitTestResult.getExtra());
         tv.setTextColor(0x8F000000);
         tv.setPadding(15, 5, 15, 5);
         menu.setHeaderView(tv);
         }
         }
         });

         setDownloadListener(new MyDownLoadListener(mContext));*/
    }

    /**
     * 不会自动调用，如果有默认style时，在第二个构造函数中调用
     * @param context
     * @param attrs
     * @param defStyleAttr
     */
    public BaseWebView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    /*public void setLastUrl(String lastUrl) {
     this.mszLastUrl = lastUrl;
     }*/

    public String getLastUrl() {
        return _mszLastUrl;
    }

    @Override
    public void loadUrl(String url) {
        Pattern patternHttp = Pattern.compile("(?i)^http[s]{0,1}://", Pattern.CASE_INSENSITIVE);
        Matcher matcherHttp = patternHttp.matcher(url);
        if (matcherHttp.matches()) {
            // 加载普通 Http 协议访问
            stopLoading();
            loadUrl(url);
            return;
        } else {
            // 加载其他协议类型访问
            Pattern pattern = Pattern.compile("^[a-z]+:.*", Pattern.MULTILINE);
            Matcher matcher = pattern.matcher(url);
            if (matcher.find()) {
                super.loadUrl(url);
                return;
            } 
        }
    }

    public void setFilePathCallback(ValueCallback<Uri[]> mFilePathCallback) {
        this.mFilePathCallback = mFilePathCallback;
    }

    public ValueCallback<Uri[]> getFilePathCallback() {
        return mFilePathCallback;
    }

    public void setFileChooserParams(WebChromeClient.FileChooserParams mFileChooserParams) {
        this.mFileChooserParams = mFileChooserParams;
    }

    public WebChromeClient.FileChooserParams getFileChooserParams() {
        return mFileChooserParams;
    }

    private class MyWebChromeClient extends WebChromeClient {
        @Override
        public void onProgressChanged(WebView view, int newProgress) {
            //super.onProgressChanged(view, newProgress);
        }

        @Override
        public boolean onShowFileChooser(WebView webView, ValueCallback<Uri[]> filePathCallback, WebChromeClient.FileChooserParams fileChooserParams) {
            setFilePathCallback(filePathCallback);
            setFileChooserParams(fileChooserParams);
            //ToastUtils.show("onShowFileChooser");
            Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
            intent.setType("*/*");
            ((AppCompatActivity)mContext).startActivityForResult(intent, MainActivity.REQUEST_FILE_CHOOSER);
            return true;
            //return super.onShowFileChooser(webView, filePathCallback, fileChooserParams);
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        mGestureDetector.onTouchEvent(event);
        return super.onTouchEvent(event);
    }

    class BaseWebViewHandler extends Handler {

        public static final String TAG = "BaseWebViewHandler";

        public static final int MSG_RELOAD_LOG = 0X100;
        public static final int MSG_SHOW_DATA = 0X123;

        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_SHOW_DATA:{
                        //LogUtils.d(TAG, "(String)msg.obj is " + (String)msg.obj);
                        loadDataWithBaseURL("", (String)msg.obj, "text/html", "UTF-8", "");
                        break;
                    }
            }
            super.handleMessage(msg);
        }
    }

    /**
     * 逻辑处理
     * @author linzewu
     */
    final class InJavaScriptLocalObj {
        @JavascriptInterface
        public void getSource(String html) {
            //LogUtils.d(TAG, html);
        }
    }

    public class MyDownLoadListener implements DownloadListener {
        private Context mContext;

        public MyDownLoadListener(Context context) {
            this.mContext = context;
        }

        @Override
        public void onDownloadStart(String url, String userAgent, String contentDisposition, String mimetype, long contentLength) {
            Uri uri = Uri.parse(url);
            Intent intent = new Intent(Intent.ACTION_VIEW, uri);
            mContext.startActivity(intent);
        }
    }

    /**
     * @Author ZhanGSKen@QQ.COM
     * @Date 2023/06/22 12:57:08
     * @Describe 网页客户端基础类
     */
    class BaseWebViewClient extends WebViewClient {

        public static final String TAG = "BaseWebViewClient";

        @Override
        public void onPageFinished(WebView view, String url) {
            view.loadUrl("javascript:window.local_obj.showSource('<head>'+" +
                         "document.getElementsByTagName('html')[0].innerHTML+'</head>');");
            super.onPageFinished(view, url);
            mIOnPageFinished.onPageFinished(url);
            LogUtils.d(TAG, "Page load finished : " + url);
            _mszLastUrl = url;
            //ToastUtils.show("onPageFinished");

            new Handler(Looper.getMainLooper()).postDelayed(new Runnable(){
                    @Override
                    public void run() {
                        if (mAuthLoginDialog != null) {
                            mAuthLoginDialog.checkAndSaveLastAuthenticationBean();
                            mAuthLoginDialog = null;
                        }
                    }
                }, 1000);

        }

        @Override
        public void onReceivedSslError(WebView view, SslErrorHandler handler, SslError error) {
            // WinBoll Studio 网站证书校验
            /*if (SSLUtil.isSSLCertOk(error.getCertificate(), "9F568FCA9E2B3D59FF3BE2F5FF77CB5BFE1D30562D9880D2D354F0DF63A012E1")) {
             LogUtils.d(TAG, "WinBoll Studio 网站证书校验通过。");
             handler.proceed();
             return;
             }*/
            LogUtils.d(TAG, "onReceivedSslError 0\nerror : " + error.toString());
            //LogUtils.d(TAG, "On Received Ssl Error.");
            //handler.proceed();

            //super.onReceivedSslError(view, handler, error);
        }

        @Override
        public void onReceivedHttpError(WebView view, WebResourceRequest request, WebResourceResponse errorResponse) {
            int statusCode = errorResponse.getStatusCode();
            /*if (statusCode == 404) {
             // 处理404错误
             Log.d("WebView", "Received a 404 error: " + failingUrl);
             }*/
            LogUtils.d(TAG, "onReceivedHttpError statusCode " + Integer.toString(statusCode));
            super.onReceivedHttpError(view, request, errorResponse);
        }

        @Override
        public void onReceivedError(WebView view, int errorCode,
                                    String description, String failingUrl) {
            super.onReceivedError(view, errorCode, description, failingUrl);
            /*if (errorCode == WebViewClient.ERROR_TIMEOUT) {
             LogUtils.d(TAG, "WebViewClient.ERROR_TIMEOUT");
             ToastUtils.show("WebViewClient.ERROR_TIMEOUT");
             } else if (errorCode == WebViewClient.ERROR_UNKNOWN) {
             LogUtils.d(TAG, "WebViewClient.ERROR_UNKNOWN");
             ToastUtils.show("WebViewClient.ERROR_UNKNOWN");
             }*/
            LogUtils.d(TAG, "onReceivedError 1\nerrorCode : " + errorCode
                       + "\ndescription :" + description + "\nfailingUrl : " + failingUrl);

            //在此处显示加载失败页面
            //loadFailure.setVisibility(View.VISIBLE);
        }

        @Override
        public void onReceivedError(WebView view, WebResourceRequest request, WebResourceError error) {
            super.onReceivedError(view, request, error);
            LogUtils.d(TAG, "onReceivedError 2\nUrl : " + request.getUrl()
                       + "\nErrorCode : " + error.getErrorCode()
                       + "\nDescription : " + error.getDescription());
        }

        @Override
        public void onReceivedLoginRequest(WebView view, String realm, String account, String args) {
            super.onReceivedLoginRequest(view, realm, account, args);
            LogUtils.d(TAG, "onReceivedLoginRequest");
        }

        @Override
        public void onLoadResource(WebView view, String url) {
            super.onLoadResource(view, url);
            //加载资源
        }

        @Override
        public void onReceivedHttpAuthRequest(WebView view,
                                              HttpAuthHandler handler, String host, String realm) {
            //super.onReceivedHttpAuthRequest(view, handler, host, realm);
            mAuthLoginDialog = new AuthLoginDialog(view, handler,  host, realm);
            mAuthLoginDialog.show();
        }

        @Override
        public void onPageStarted(WebView view, String url, Bitmap favicon) {
            super.onPageStarted(view, url, favicon);
            //开始加载页面
            LogUtils.d(TAG, "Page load started : " + url);
            _mszLastUrl = url;
        }

        @Override
        public boolean shouldOverrideUrlLoading(WebView view, String url) {
            //点击链接跳转，对网页中超链接按钮的响应。 
            //view.loadUrl(url);
            return true;
        }
    }

    public interface IOnPageFinished {
        void onPageFinished(String url);
    }

    public void setOnPageFinished(IOnPageFinished iOnPageFinished) {
        mIOnPageFinished = iOnPageFinished;
    }
}
